#!/usr/bin/env ruby -wKU
#
#  exceLogger
#
#  Created by Gregory Clarke on 6/30/09.
#  Copyright (c) 2009 Intelligent Assistance, Inc. All rights reserved.
#

require 'exceLogger_algorithm.bundle'

class AppController < OSX::NSObject

	ib_outlets :mainWindow, :deactivateMenuItem
	ib_outlets :exceLoggerView, :chooseExcelButton, :excelIcon, :excelFileName, :chooseXMLButton, :xmlIcon, :xmlFileName, :saveXMLButton, :exceLoggerSpinner, :onSaveRadioButtons
	ib_outlets :progressWindow, :progressView, :progressIndicator, :progressMessage
	ib_outlets :regoView, :deactivateAreaView, :webView, :problemMessage, :closeRegoButton
	
	def awakeFromNib
		puts "awakeFromNib"
		@mainWindow.setTitle("exceLogger")
		@excelIcon.setHidden(true)
		@excelIcon.setToolTip("")
		@excelFileName.setStringValue("")
		@xmlIcon.setHidden(true)
		@xmlIcon.setToolTip("")
		@xmlFileName.setStringValue("")
		@excel_file = nil
		@xml_file = nil
		# check for Application Support folder
		paths = OSX::NSSearchPathForDirectoriesInDomains(OSX::NSApplicationSupportDirectory, OSX::NSUserDomainMask, true)
		if paths.length > 0
			ENV['ROO_TMP'] = paths[0].stringByExpandingTildeInPath.to_s
		end
		@webView.setDrawsBackground(false)
		check_for_expired_beta()
	end

	### rego methods ###
	
	def applicationWillFinishLaunching(sender) # applicationWillFinishLaunching: is sent before application:openFile: (so it can deal with a file dropped onto the app icon)
		puts "applicationWillFinishLaunching"
		check_for_expired_beta()
		if @this_rego[0]["active"]
			@deactivateMenuItem.setEnabled(true)
			showExceLoggerView()
		elsif @this_rego[0]["mac_problem"]==true
			check_for_rego(@this_rego)
		else
			@deactivateMenuItem.setEnabled(false)
			displayDemoSheet()
		end
	end
	
	def displayDemoSheet
		report = OSX::NSAlert.alloc.init
		report.setMessageText("#{SKU} has not been activated \nfor this machine.")
		report.setAlertStyle(OSX::NSInformationalAlertStyle)
		report.setInformativeText("Would you like to activate #{SKU}, or try it in demo mode (which is limited to 10 clips)?")
		report.addButtonWithTitle("Activate")
		report.addButtonWithTitle("Demo")
		report.beginSheetModalForWindow_modalDelegate_didEndSelector_contextInfo(@mainWindow, self, "didEndDemoSheet:returnCode:contextInfo:", nil)
	end

	def didEndDemoSheet_returnCode_contextInfo(sheet, buttonClicked, info)
		sheet.window.orderOut(nil)
		if buttonClicked==OSX::NSAlertFirstButtonReturn
			newFrame = self.newFrameForNewContentView(@regoView)
			showActivationView(newFrame, WEB_APP + "license/" + SKU.gsub(" ", "%20") + "?sn=" + @this_rego[0]["number"].to_s + "&mac=" + MAC) # NEW
		else
			puts "DEMO MODE"
			showExceLoggerView()
		end
	end

	def showDeactivation
		newFrame = self.newFrameForNewContentView(@deactivateAreaView) # 500 x 200
		@mainWindow.setMinSize(OSX::NSSize.new(500, 200+22))
		@mainWindow.setMaxSize(OSX::NSSize.new(500, 200+22))
		@mainWindow.setContentView(@deactivateAreaView)
		showActivationView(newFrame, WEB_APP + "destroy/" + @this_rego[0]["number"].to_s)
		@closeRegoButton.setTitle("Cancel")
	end
	ib_action :showDeactivation # exceLogger > Deactivate
	
	def showActivationView(frame, url)
		puts "showActivationView #{url}"
		@mainWindow.setFrame_display frame, true
		@mainWindow.setContentView(@regoView)
		@mainWindow.setTitle("Activation")
		# notification
		center = OSX::NSNotificationCenter.defaultCenter
		center.addObserver_selector_name_object(self, :webViewProgressFinished, "WebProgressFinishedNotification", @webView) # name is WebProgressFinishedNotification NOT WebViewProgressFinishedNotification
		@webView.mainFrame.loadRequest( OSX::NSURLRequest.requestWithURL( OSX::NSURL.URLWithString(url) ) )
	end
	
	def webViewProgressFinished(notification)
		puts notification.name
		if notification.name=="WebProgressFinishedNotification"
			if @webView.mainFrame.dataSource.nil? # in case page doesn't load...
				@webView.mainFrame.stopLoading
				@problemMessage.setHidden(false)
				@webView.setHidden(true)
			else
				get_web_app_response(@webView.mainFrame.dataSource.response.URL)
			end
		end
	end 
	
	def closeActivationView
		if @closeRegoButton.title=="Continue" || @closeRegoButton.title=="Cancel"
			showExceLoggerView()
		else
			OSX::NSApplication.sharedApplication.terminate(self)
		end
	end
	ib_action :closeActivationView # on click of Registration window's Continue button
	
	### end rego methods ###
	
	def showExceLoggerView
		puts "showExceLoggerView"
		setupView(@exceLoggerView, "exceLogger for FCP", 500, 280)
	end

	def setupView(view, title, width, height)
		@mainWindow.setTitle(title)
		oldFrame = @mainWindow.frame
		newFrame = self.newFrameForNewContentView(view)
		@mainWindow.setMinSize(OSX::NSSize.new(width, height+22))
		@mainWindow.setMaxSize(OSX::NSSize.new(width, 200+22))
		@mainWindow.setFrame_display_animate(newFrame, true, true)
		@mainWindow.setContentView(view)
	end

	def newFrameForNewContentView(view)
		newFrameRect = @mainWindow.frameRectForContentRect(view.frame)
		oldFrameRect = @mainWindow.frame
		newSize = newFrameRect.size
		oldSize = oldFrameRect.size
		frame = @mainWindow.frame
		frame.size = newSize
		frame.origin.y = frame.origin.y - (newSize.height - oldSize.height)
		frame
	end
		
	def showHelp(sender)
		openPDF("exceLogger - Help.pdf")
	end
	ib_action :showHelp # Help > exceLogger Help
	
	def openPDF(filename)
		path_to_file = __FILE__.sub("AppController.rb",filename)
		system('open', path_to_file)
	end
	
	def visitWebsite(sender)
		system('open', 'http://www.AssistedEditing.com/')
	end
	ib_action :visitWebsite # Help > Visit www.AssistedEditing.com

	def sendFeedback(sender)
		OSX::JRFeedbackController.showFeedback
	end
	ib_action :sendFeedback # exceLogger > Provide Feedback
	
	### exceLogger methods ###
	
	def chooseExcelFile
		@saveXMLButton.setEnabled(false)
		@excel_file = nil
		@excelIcon.setHidden(true)
		@excelIcon.setToolTip("")
		@excelFileName.setStringValue("")
		filetypes = ["xls","xlsx"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setTitle("Choose the Excel document to open")
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndLoggerOpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseExcelFile
	
	def chooseBatchListXmlFile
		@saveXMLButton.setEnabled(false)
		@xml_file = nil
		@xmlIcon.setHidden(true)
		@xmlIcon.setToolTip("")
		@xmlFileName.setStringValue("")
		filetypes = ["xml"]
		oPanel = OSX::NSOpenPanel.openPanel
		oPanel.setTitle("Choose the exported Bin(s) of Clips to process (XML file)")
		oPanel.setAllowsMultipleSelection(false)
		oPanel.beginSheetForDirectory_file_types_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, filetypes, @mainWindow, self, 'didEndLoggerOpenSheet:returnCode:contextInfo:', nil)
	end
	ib_action :chooseBatchListXmlFile
	
	def didEndLoggerOpenSheet_returnCode_contextInfo(oPanel, buttonClicked, info)
		if buttonClicked == OSX::NSOKButton
			oPanel.orderOut(self)
			file = oPanel.filenames.objectAtIndex(0)
			application_setLoggerFile(self, file)
		end
	end
	
	def application_setLoggerFile(sender, path)
		OSX::NSDocumentController.sharedDocumentController().noteNewRecentDocumentURL( OSX::NSURL.fileURLWithPath(path) ) # add this file to the Recent menu
		fileManager = OSX::NSFileManager.defaultManager
		chosen_file = path.to_s
		chosen_file_extension = File.extname(chosen_file)
		if chosen_file_extension==".xls" || chosen_file_extension==".xlsx"
			@excel_file = chosen_file
			@excelIcon.setHidden(false)
			@excelIcon.setToolTip(path.to_s)
			@excelFileName.setStringValue(File.basename(chosen_file))
		elsif chosen_file_extension==".xml"
			@xml_file = chosen_file
			@xmlIcon.setHidden(false)
			@xmlIcon.setToolTip(path.to_s)
			@xmlFileName.setStringValue(File.basename(chosen_file))
		else
			return false
		end
		if @excel_file!=nil && @xml_file!=nil
			@saveXMLButton.setEnabled(true)
		end
	end
	
	def saveLoggedXml
		@progressIndicator.setIndeterminate(true)
		@progressIndicator.setUsesThreadedAnimation(true)
		@progressIndicator.startAnimation(self)
		@progressMessage.setStringValue("Adding log notes...")
		puts "on save radio buttons:", @onSaveRadioButtons.selectedCell.tag.inspect
		if @onSaveRadioButtons.selectedCell.tag==0 # save XML file
			sPanel = OSX::NSSavePanel.savePanel
			sPanel.setTitle("Save the new XML file")
			sPanel.setRequiredFileType("xml")
			sPanel.setExtensionHidden(false)
			sPanel.setCanSelectHiddenExtension(true)
			sPanel.beginSheetForDirectory_file_modalForWindow_modalDelegate_didEndSelector_contextInfo(nil, nil, @mainWindow, self, 'didEndSaveLoggedXml:returnCode:contextInfo:', nil)
		else # save temp and open into FCP project
			begin
				displayProgressSheet()
				add_log_notes(@excel_file, @xml_file, TEMP_XML_FILE)
				@progressMessage.setStringValue("Switching to Final Cut Pro\n")
				@progressView.displayIfNeeded
				fileManager = OSX::NSFileManager.defaultManager
				fileAttributes = { :NSFileExtensionHidden => 1, :NSFileHFSCreatorCode => 1264941383 } # extension is hidden, Final Cut Pro is creator code
				fileManager.setAttributes_ofItemAtPath_error(fileAttributes, TEMP_XML_FILE, nil)
				sleep 2
				OSX::NSWorkspace.sharedWorkspace.openFile_withApplication(TEMP_XML_FILE, "Final Cut Pro")
				didEndProgressSheet_returnCode_contextInfo(@progressWindow, 2, nil)
			rescue
				didEndProgressSheet_returnCode_contextInfo(@progressWindow, -1, nil)
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was a problem creating this XML file.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				puts "RESCUE!!! Problem with explore_metadata()"
				alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
			end
		end
	end
	ib_action :saveLoggedXml
	
	def didEndSaveLoggedXml_returnCode_contextInfo(sPanel, buttonClicked, info)
		########## SAVE LOGGED XML ##########
		if buttonClicked == OSX::NSOKButton
			sPanel.orderOut(self)
			if sPanel.isExtensionHidden?
				extension_hidden = 1
			else
				extension_hidden = 0
			end
			puts "is extension hidden?", sPanel.isExtensionHidden
			new_xml_file = sPanel.filenames.objectAtIndex(0).to_s
			displayProgressSheet()
			begin
				add_log_notes(@excel_file, @xml_file, new_xml_file)
				fileManager = OSX::NSFileManager.defaultManager
				fileAttributes = { :NSFileExtensionHidden => 1, :NSFileHFSCreatorCode => 1264941383 } # extension is hidden, Final Cut Pro is creator code
				fileManager.setAttributes_ofItemAtPath_error(fileAttributes, new_xml_file, nil)
				didEndProgressSheet_returnCode_contextInfo(@progressWindow, 1, nil)
			rescue
				didEndProgressSheet_returnCode_contextInfo(@progressWindow, -1, nil)
				alert = OSX::NSAlert.alloc.init
				alert.setMessageText("There was a problem saving this XML file.")
				alert.setAlertStyle(OSX::NSInformationalAlertStyle)
				puts "RESCUE!!! Problem with explore_metadata()"
				alert.setInformativeText("Please choose another XML file exported from\nFinal Cut Pro.")
				alert.addButtonWithTitle("OK")
				alert.runModal
			end
		end
	end

	### end exceLogger methods ###

	def displayProgressSheet
		OSX::NSApp.beginSheet_modalForWindow_modalDelegate_didEndSelector_contextInfo(@progressWindow, @mainWindow, self, 'didEndProgressSheet:returnCode:contextInfo:', nil)
		@progressView.displayIfNeeded
	end

	def didEndProgressSheet_returnCode_contextInfo(sheet, code, info)
		puts "didEndProgressSheet_returnCode_contextInfo code #{code}"
		sheet.orderOut(nil) # or self?
		@progressMessage.setStringValue("")
		@progressIndicator.stopAnimation(self)
		OSX::NSApp.endSheet(sheet)
	end

	def applicationShouldTerminateAfterLastWindowClosed(sender)
		true
	end

end
